package domain;

import java.util.*;

import test.*;

import junit.framework.*;

public class PlayerTest extends TestCase {
   private final static String NAME = "x";
   static final int INITIAL_STAKE = 1000;
   private Player player;

   protected void setUp() {
      player = new Player(NAME);
   }

   public static Player createPlayer(int playerNumber) {
      return new Player("" + playerNumber, INITIAL_STAKE);
   }

   public static void assertPlayer(int playerNumber, Player player) {
      assertEquals("" + playerNumber, player.getName());
   }

   public void testCreate() {
      assertEquals(NAME, player.getName());
      assertEquals(0, player.chipCount());
   }

   public void testCreateWithInitialStake() {
      player = new Player(NAME, INITIAL_STAKE);
      assertEquals(NAME, player.getName());
      assertEquals(INITIAL_STAKE, player.chipCount());
   }

   public void testHoleCards() {
      dealHoleCards();
      List<Card> holeCards = player.holeCards();
      CollectionsTestUtil.assertCollection(holeCards, CardTest.CARD1,
            CardTest.CARD2);
   }

   public void testRemoveCards() {
      dealHoleCards();
      player.removeCards();
      assertTrue(player.holeCards().isEmpty());
   }

   private void dealHoleCards() {
      player.dealToHole(CardTest.CARD1);
      player.dealToHole(CardTest.CARD2);
   }

   public void testBankroll() {
      assertEquals(0, player.chipCount());
      player.add(1000);
      assertEquals(1000, player.chipCount());
      player.bet(200);
      assertEquals(800, player.chipCount());
   }

   public void testInsufficientFunds() {
      try {
         player.bet(1);
         fail("expected insuff. funds exception");
      }
      catch (InsufficientFundsException expected) {
         assertEquals(0, player.chipCount());
      }
   }
}