package domain;

import java.util.*;

public class Ring implements Iterable<Player> {
   private List<Player> players = new ArrayList<Player>();
   private int button;

   public List<Player> players() {
      return players;
   }

   public void add(Player player) {
      players.add(player);
   }

   public void add(Player... players) {
      for (Player player: players)
         this.players.add(player);
   }

   public void setButton(int button) {
      if (button > players.size())
         throw new InvalidButtonPositionException(button);
      this.button = button;
   }

   public int buttonPosition() {
      return button;
   }

   public Player smallBlind() {
      return getPlayer(button + 1);
   }

   public Player bigBlind() {
      return getPlayer(button + 2);
   }

   public Player getPlayer(int position) {
      return players.get(ringPosition(position) - 1);
   }

   private int ringPosition(int position) {
      if (position > players.size())
         return position - players.size();
      return position;
   }

   public void advanceButton() {
      button = ringPosition(button + 1);
   }

   public void removeAllCards() {
      for (Player player: players)
         player.removeCards();
   }

   public void collectBlinds(int small, int big) {
      smallBlind().bet(small);
      bigBlind().bet(big);
   }

   public Iterator<Player> iterator() {
      return new PlayerIterator();
   }

   class PlayerIterator implements Iterator<Player> {
      private int currentPosition;
      private int iterations = 0;

      public PlayerIterator() {
         currentPosition = button + 1;
      }

      public boolean hasNext() {
         return iterations < players.size();
      }

      public Player next() {
         iterations++;
         return getPlayer(currentPosition++);
      }

      public void remove() {
      }
   }
}