package domain;

import java.util.*;

import test.*;

import junit.framework.*;

public class RingTest extends TestCase {
   private Ring ring;
   private Player player1;
   private Player player2;
   private Player player3;

   protected void setUp() {
      ring = new Ring();
      player1 = PlayerTest.createPlayer(1);
      player2 = PlayerTest.createPlayer(2);
      player3 = PlayerTest.createPlayer(3);
   }

   public void testCreate() {
      assertEquals(0, ring.players().size());
   }

   public void testAddPlayer() {
      addOnePlayer();
      CollectionsTestUtil.assertCollection(ring.players(), player1);
   }

   public void testAddMultiplePlayersAtOnce() {
      ring.add(player1, player2);
      CollectionsTestUtil.assertCollection(ring.players(), player1, player2);
   }

   public void testButtonPosition() {
      assertEquals(0, ring.buttonPosition());
      ring.add(player1);
      ring.setButton(1);
      assertEquals(1, ring.buttonPosition());
   }

   public void testInvalidButtonPositionNoPlayers() {
      try {
         ring.setButton(1);
         fail("expected exception");
      }
      catch (InvalidButtonPositionException expected) {
         assertEquals(1, expected.getPosition());
      }
   }

   public void testInvalidButtonPosition() {
      addTwoPlayers();
      try {
         ring.setButton(3);
         fail("expected exception");
      }
      catch (InvalidButtonPositionException expected) {
         assertEquals(3, expected.getPosition());
      }
   }

   public void testBlindPlayers() {
      addThreePlayers();

      ring.setButton(3);
      PlayerTest.assertPlayer(1, ring.smallBlind());
      PlayerTest.assertPlayer(2, ring.bigBlind());

      ring.advanceButton();
      PlayerTest.assertPlayer(2, ring.smallBlind());
      PlayerTest.assertPlayer(3, ring.bigBlind());

      ring.advanceButton();
      PlayerTest.assertPlayer(3, ring.smallBlind());
      PlayerTest.assertPlayer(1, ring.bigBlind());
   }

   public void testExtractBlinds() {
      addTwoPlayers();

      ring.setButton(2);
      final int small = 20;
      final int big = 40;
      ring.collectBlinds(small, big);
      assertEquals(PlayerTest.INITIAL_STAKE - small, player1.chipCount());
      assertEquals(PlayerTest.INITIAL_STAKE - big, player2.chipCount());
   }

   public void testRemoveAllCards() {
      addThreePlayers();
      ring.removeAllCards();
      for (Player player: ring.players())
         assertEquals(0, player.holeCards().size());
   }

   public void testForEachInRing() {
      addThreePlayers();
      ring.setButton(2);
      List<Player> collected = new ArrayList<Player>();
      for (Player player: ring)
         collected.add(player);
      CollectionsTestUtil
            .assertCollection(collected, player3, player1, player2);
      assertEquals(3, collected.size());
   }

   private void addOnePlayer() {
      ring.add(player1);
   }

   private void addTwoPlayers() {
      addOnePlayer();
      ring.add(player2);
   }

   private void addThreePlayers() {
      addTwoPlayers();
      ring.add(player3);
   }
}